unit uCoolObject;

{
TCoolObject started as a standard Automation object created with the Delphi
"New ... Automation Object" wizard.  But then the auto-generated code was
modified to be an AutoObjectWithEvents (see uComEvents - thank you
Ryan VanIderstine).
}

interface

uses
  ComObj, AXScriptDemo_TLB, uComEvents, Windows, AXCtrls, ActiveX;

type
  TCoolObject = class(TAutoObjectWithEvents, ICoolObject)
  protected
    FCoolObjectEvents:ICoolObjectEvents;  //stores the interface to the current event sink.
    FCurrentMessage:WideString;

    //TAutoObjectWithEvents Stuff
    procedure EventSinkChanged( const Events: IUnknown ); override;
    procedure Initialize; override;

    //ICoolObject Interface
    function Get_CurrentMessage: WideString; safecall;
    procedure Set_CurrentMessage(const Value: WideString); safecall;
    procedure ShowMessage(const S: WideString); safecall;
    procedure ShowMessageNoEvent(const S: WideString); safecall;

    //Helper procedures - event firing
    procedure FireOnShowMessage(const S:WideString);
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses ComServ, Dialogs;

constructor TCoolObject.Create;
begin
  inherited Create;
  FCurrentMessage:='';
end;

destructor TCoolObject.Destroy;
begin
  inherited Destroy;
end;

procedure TCoolObject.EventSinkChanged( const Events: IUnknown );
begin
  FCoolObjectEvents := Events as ICoolObjectEvents;
end;

procedure TCoolObject.Initialize;
begin
  inherited;
  FUseDispatchSilencer:=true;
end;

function TCoolObject.Get_CurrentMessage: WideString;
begin
  Result:=FCurrentMessage;
end;

procedure TCoolObject.Set_CurrentMessage(const Value: WideString);
begin
  FCurrentMessage:=Value;
end;

procedure TCoolObject.FireOnShowMessage(const S:WideString);
begin
  if Assigned(FCoolObjectEvents) then FCoolObjectEvents.OnShowMessage(S);
end;

procedure TCoolObject.ShowMessage(const S: WideString);
begin
  ShowMessageNoEvent(S);
  
  //Uses a helper function to fire events.  Some events need to have some of
  //their parameters manipulated so that they're in an automation compatible type.
  //OnShowMessage needs no such manipulation sine it only passes a WideString.
  FireOnShowMessage(S);
end;

procedure TCoolObject.ShowMessageNoEvent(const S: WideString);
begin
  FCurrentMessage:=S;
  Dialogs.ShowMessage(S);
end;

initialization
  TAutoObjectWithEventsFactory.Create(ComServer, TCoolObject, Class_CoolObject);
end.
